/*
-------------------------------------------------------------------------------

Name: TB_SpreadLayersToPanels.js

Description: This script can take layers in a single panel and redistribute them to new panels.

Usage: Select panels, then pick the layers you want to spread across multiple panels, then pick the layers you want to keep on every panel.

Author: Corey Barnes and Karl Olson

Created: 2018/11/27

Updated: 2018/11/28

Version: 1.0.3

Social Media Info:
Corey on Twitter - http://twitter.com/myanimewaifu
Karl on Twitter - http://twitter.com/karlrolson

Contact Email: livingfruitvirus@gmail.com

-------------------------------------------------------------------------------
*/

function TB_SpreadLayersToPanels() { // BEGIN FUNCTION
  MessageLog.trace('Spread Layers to Panels--------- ');

  var selMgr = new SelectionManager();
  var layMgr = new LayerManager();
  var sbMgr = new StoryboardManager();

  var panelSel = selMgr.getPanelSelection();
  var sceneSel = selMgr.getSceneSelection();
  if (panelSel.length != 1) return; // kills script if more than one panel is selected.

  // After selecting the panels, we run the script to collect information from the user.
  // Starting with what layers they want to spread out, based on the panel they've selected.
  // First, we collect the layers.
  var layerNames = new Array(0);
  var panelId = panelSel[0]; // this is the source panel, which we will delete later.
  var sceneId = sceneSel[0]; // this is the source scene
  var numberOfLayers = layMgr.numberOfLayers(panelId);

  for (var j = 0; j < numberOfLayers; ++j) {
    var layerName = layMgr.layerName(panelId, j);

    if (layerNames.indexOf(layerName) == -1) {
      layerNames.push(layerName);
    }
  }

  // Next, we create a checkbox of all the layers they wish to spread out.
  var da = new Dialog();
  da.title = 'Spread Layers';

  var groupa = new GroupBox();
  groupa.title = 'Layers to spread out.';
  var counta = 0;
  var checkboxArray = new Array(0);

  for (var i = 0; i < layerNames.length; ++i) {
    var c = new CheckBox();
    c.text = layerNames[i];
    counta++;
    checkboxArray.push(c);
    groupa.add(c);
  }

  da.add(groupa);

  var rca = da.exec();

  // Begin separating the layers
  var layersToSpread = new Array(0);
  var layersNotToSpread = new Array(0);
  var namesNotToSpread = new Array(0);

  if (rca) {
    for (var f = 0; f < counta; f++) {
      var r = checkboxArray[f];
      if (r.checked === true) {
        layersToSpread.push(f);
      } else {
        layersNotToSpread.push(f);
        namesNotToSpread.push(r.text);
      }
    }
  } else {
    return;
  }

  var layersToPreserve = new Array(0);
  // Next, we re-create the same checkbox, but to list layers that the user wishes to keep throughout.
  if (layersNotToSpread.length > 0 && layersToSpread.length > 0) {
    db = new Dialog();
    db.title = 'Preserve Layers';

    var groupb = new GroupBox();
    groupb.title = 'Layers to retain throughout.';
    var countb = 0;
    var checkboxArrayB = new Array(0);

    for (var i = 0; i < layersNotToSpread.length; ++i) {
      var c = new CheckBox();
      c.text = namesNotToSpread[i];
      countb++;
      checkboxArrayB.push(c);
      groupb.add(c);
    }

    db.add(groupb);

    var rcb = db.exec();

    if (!rcb) {
      return;
    }

    // Now we retrieve those layers.

    if (rcb) {
      for (var f = 0; f < countb; f++) {
        var r = checkboxArrayB[f];
        if (r.checked === true) {
          layersToPreserve.push(layersNotToSpread[f]);
        }
      }
    } else {
      return;
    }
  }

  // ------ Begin running the script ------

  layersToSpread = layersToSpread.reverse(); // This changes the spreadable layers to ascending order.

  if (layersToSpread.length > 0) {
    scene.beginUndoRedoAccum('Spread Layers to Panels');

    // Duplicate one panel for every value in layersToSpread
    for (var i = 0; i < layersToSpread.length; i++) {
      Action.perform('onActionDuplicatePanel()');
    }

    // Next, built the panelCollection array.
    // Find where the source panel is located in the scene
    var numPanels = sbMgr.numberOfPanelsInScene(sceneId);
    var startPanel;
    
    if ( panelId === sbMgr.panelInScene(sceneId, 0)) {
        startPanel = 0;
	} else {
    for (var i = 1; i <= numPanels; i++) {
      var p = sbMgr.panelInScene(sceneId, i);
      if (p === panelId) {
        startPanel = i;
        break;
      }
    }
}
	
    // Use the number of duped panels to track down the additional panel IDs
    var panelCollection = new Array(0);
    var totalPanels = startPanel + layersToSpread.length;

    for (var i = startPanel + 1; i <= totalPanels; i++) {
      var addPanel = sbMgr.panelInScene(sceneId, i);
      panelCollection.push(addPanel);
    }

    // Begin the process of running through each panel and deleting the layers accordingly
    // i = integer for the panel to select, and synchronized with the layersToSpread array
    for (var i = 0; i < panelCollection.length; i++) {
      var currentPanel = panelCollection[i];
      var layersNotToDelete = new Array(0);
      layersNotToDelete.push(layersToSpread[i]);

      for (var f = 0; f < layersToPreserve.length; f++) {
        layersNotToDelete.push(layersToPreserve[f]);
      }

      // Begin going through each layer
      // n = integer representing the layer and whether or not we should kill that layer
      for (var n = 0; n < layMgr.numberOfLayers(currentPanel); n++) {
        if (layersNotToDelete.indexOf(n) === -1) {
          layMgr.deleteLayer(currentPanel, n);
          n--;
          // deleting layers changes the length of layers in the panel
          // so we have to update the layer not to delete indexes to reflect a shorter array
          for (var o = 0; o < layersNotToDelete.length; o++) {
            if (layersNotToDelete[o] >= n) {
              layersNotToDelete[o] = layersNotToDelete[o] - 1;
            }
          }
        }
      } // End going through each layer
    } // End of for loop for panels

    var dc = new Dialog();
    dc.title = 'Delete Original Panel?';
    dc.okButtonText = 'Yes';
    dc.cancelButtonText = 'No';

    var dcLabel = new Label;
    dcLabel.text = "Do you want to delete the original source panel?";

    dc.add(dcLabel);

    var dca = dc.exec();

    if (dca) {
      sbMgr.deletePanel(panelId);
    }

    scene.endUndoRedoAccum();
    // Prepare report.
    MessageBox.information('Layers successfully spread.');
  } else {
    MessageBox.information('No layers selected to spread');
  }
}
